/*
    Compilation: g++ -std=c++17 fruits.cpp -o fruits
    Execution: ./fruits apple apple kiwi apple lemon pomegranate apple lemon kiwi
 */

#include <iostream>
#include <vector>

enum Color {Red, Orange, Yellow, Green, Blue, Purple};

struct Fruit {
    const char *name;
    Color color;
    float cost;
};

typedef struct Fruit * fruit_t;

typedef std::vector<fruit_t> fruityVector;

fruit_t newFruit(const char *name, Color color, float cost) {
    fruit_t F = new struct Fruit;
    F->name = name;
    F->color = color;
    F->cost = cost;
    return F;
}

void destroyFruit(fruit_t F) {
    delete F;
}

float basketCost(fruityVector B) {
    float result = 0;
    for (fruit_t F : B) {
        result += F->cost;
    }
    return result;
}

void printer(fruit_t F) {
    std::cout << "(" << F->name << " $" << F->cost << ")";
}

void printer(fruityVector B) {
    std::cout << "< ";
    for (fruit_t F: B) {
        printer(F);
        std::cout << " ";
    }
    std::cout << ">\n";
}

int main(int argc, char **argv) {
    fruit_t A = newFruit("apple", Red, 1.32);
    fruit_t O = newFruit("orange", Red, 1.45);
    fruit_t L = newFruit("lemon", Yellow, 0.62);
    fruit_t K = newFruit("kiwi", Green, 2.19);
    fruit_t P = newFruit("pomegranate", Red, 5.92);

    fruityVector catalog;
    catalog.push_back(A);
    catalog.push_back(O);
    catalog.push_back(L);
    catalog.push_back(K);
    catalog.push_back(P);

    fruityVector basket;
    for (int i = 1; i < argc; i++) {
        for (fruit_t F : catalog) {
            if (strcmp(F->name, argv[i]) == 0) {
                basket.push_back(F);
            }
        }
    }

    int redCount = 0;
    for (fruit_t F : basket) {
        if (F->color == Red) {
            redCount++;
        }
    }

    std::cout << "Basket: ";
    printer(basket);
    std::cout << "Red Fruits: " << redCount << "\n";
    std::cout << "Size: " << basket.size() << "\n";
    std::cout << "Cost: $" << basketCost(basket) << "\n";

    for (fruit_t F : catalog) {
        destroyFruit(F);
    }
    catalog.clear();
    basket.clear();

}